@@ -253,12 +253,16 @@ private:
     SetDefaultParameterInt("inverse.ssrate", 16);
     SetMinimumParameterIntValue("inverse.ssrate", 1);
 
+    AddRAMParameter();
+
     // Doc example
     SetDocExampleParameterValue("io.inleft", "wv2_xs_left.tif");
     SetDocExampleParameterValue("io.inright", "wv2_xs_left.tif");
     SetDocExampleParameterValue("io.outleft", "wv2_xs_left_epi_field.tif");
     SetDocExampleParameterValue("io.outright", "wv2_xs_right_epi_field.tif");
     SetDocExampleParameterValue("epi.elevation.default", "400");
+    
+    SetMultiWriting(true);
 
     SetOfficialDocLink();
   }
