@@ -411,6 +411,12 @@ void Application::SetParameterFloat(std::string const& key, float value, bool ha
   this->SetParameterUserValue(key, hasUserValueFlag);
 }
 
+void Application::SetParameterDouble(std::string const& key, double value, bool hasUserValueFlag)
+{
+  GetParameterByKey(key)->FromDouble(value);
+  this->SetParameterUserValue(key, hasUserValueFlag);
+}
+
 void Application::SetParameterString(std::string const& parameter, std::string value, bool hasUserValueFlag)
 {
   GetParameterByKey(parameter)->FromString(value);
@@ -694,9 +700,8 @@ int Application::Execute()
   std::vector<std::string>         paramList = GetParametersKeys(true);
   int                              status    = 0;
   std::unordered_set<Application*> targetApps;
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    std::string          key      = *it;
     Parameter*           param    = GetParameterByKey(key);
     InputImageParameter* imgParam = dynamic_cast<InputImageParameter*>(param);
 
@@ -729,9 +734,8 @@ int Application::Execute()
     // Call target Execute()
     status = status | app->Execute();
   }
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    std::string          key      = *it;
     Parameter*           param    = GetParameterByKey(key);
     InputImageParameter* imgParam = dynamic_cast<InputImageParameter*>(param);
     if (imgParam)
@@ -802,9 +806,8 @@ int Application::Execute()
 
   bool UseSpecificSeed = false;
 
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    std::string key = *it;
     if ((key.compare(0, 4, "rand") == 0) && HasValue("rand"))
     {
       UseSpecificSeed         = true;
@@ -828,14 +831,14 @@ int Application::Execute()
   m_ExecuteDone   = true;
 
   // Ensure that all output image parameter have called UpdateOutputInformation()
-  for (auto it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    OutputImageParameter* outImgParamPtr = dynamic_cast<OutputImageParameter*>(GetParameterByKey(*it));
+    OutputImageParameter* outImgParamPtr = dynamic_cast<OutputImageParameter*>(GetParameterByKey(key));
     // If this is an OutputImageParameter
     if (outImgParamPtr != nullptr)
     {
       // If the parameter is enabled
-      if (IsParameterEnabled(*it))
+      if (IsParameterEnabled(key))
       {
         // Call UpdateOutputInformation()
         outImgParamPtr->GetValue()->UpdateOutputInformation();
@@ -853,10 +856,8 @@ void Application::WriteOutput()
   // writer if a RAMParameter is set
   bool         useRAM = false;
   unsigned int ram    = 0;
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    std::string key = *it;
-
     if (GetParameterType(key) == ParameterType_RAM && IsParameterEnabled(key))
     {
       Parameter*    param    = GetParameterByKey(key);
@@ -868,10 +869,16 @@ void Application::WriteOutput()
       }
     }
   }
-
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  
+  otb::MultiImageFileWriter::Pointer multiWriter;
+  if (m_MultiWriting)
+    {
+    multiWriter = otb::MultiImageFileWriter::New();
+    multiWriter->SetAutomaticStrippedStreaming(ram);
+    }
+  
+  for (auto const & key : paramList)
   {
-    std::string key = *it;
     if (GetParameterType(key) == ParameterType_OutputImage && IsParameterEnabled(key) && HasValue(key))
     {
       Parameter*            param       = GetParameterByKey(key);
@@ -888,11 +895,16 @@ void Application::WriteOutput()
         {
           outputParam->SetRAMValue(ram);
         }
-        outputParam->InitializeWriters();
+
+        outputParam->InitializeWriters(multiWriter);
         std::ostringstream progressId;
-        progressId << "Writing " << outputParam->GetFileName() << "...";
-        AddProcess(outputParam->GetWriter(), progressId.str());
-        outputParam->Write();
+        
+        if (!outputParam->IsMultiWritingEnabled())
+        {
+          progressId << "Writing " << outputParam->GetFileName() << "...";
+          AddProcess(outputParam->GetWriter(), progressId.str());
+          outputParam->Write();
+        }
       }
     }
     else if (GetParameterType(key) == ParameterType_OutputVectorData && IsParameterEnabled(key) && HasValue(key))
@@ -909,6 +921,14 @@ void Application::WriteOutput()
       }
     }
   }
+  
+  if (multiWriter && multiWriter->GetNumberOfInputs() > 0)
+  {
+    std::ostringstream progressId;
+    progressId << "Writing " << multiWriter->GetNumberOfInputs() << " output images ...";
+    AddProcess(multiWriter, progressId.str());
+    multiWriter->Update();
+  }
 }
 
 int Application::ExecuteAndWriteOutput()
@@ -1097,6 +1117,13 @@ void Application::SetDefaultParameterInt(std::string const& parameter, int value
     if (!hasUserValue)
       paramFloat->SetValue(static_cast<float>(value));
   }
+  else if (dynamic_cast<DoubleParameter*>(param))
+  {
+    DoubleParameter* paramDouble = dynamic_cast<DoubleParameter*>(param);
+    paramDouble->SetDefaultValue(static_cast<double>(value));
+    if (!hasUserValue)
+      paramDouble->SetValue(static_cast<double>(value));
+  }
   else if (dynamic_cast<RAMParameter*>(param))
   {
     RAMParameter* paramRAM = dynamic_cast<RAMParameter*>(param);
@@ -1125,6 +1152,11 @@ int Application::GetDefaultParameterInt(std::string const& parameter)
     FloatParameter* paramFloat = dynamic_cast<FloatParameter*>(param);
     ret                        = paramFloat->GetDefaultValue();
   }
+  else if (dynamic_cast<DoubleParameter*>(param))
+  {
+    DoubleParameter* paramDouble = dynamic_cast<DoubleParameter*>(param);
+    ret                        = paramDouble->GetDefaultValue();
+  }
   else if (dynamic_cast<RAMParameter*>(param))
   {
     RAMParameter* paramRAM = dynamic_cast<RAMParameter*>(param);
@@ -1154,6 +1186,23 @@ float Application::GetDefaultParameterFloat(std::string const& key)
   return param->GetDefaultValue();
 }
 
+void Application::SetDefaultParameterDouble(std::string const& key, double value)
+{
+  auto param = downcast_check<DoubleParameter>(GetParameterByKey(key));
+  param->SetDefaultValue(value);
+
+  if (!param->HasUserValue())
+  {
+    param->SetValue(value);
+  }
+}
+
+double Application::GetDefaultParameterDouble(std::string const& key)
+{
+  auto param = downcast_check<DoubleParameter>(GetParameterByKey(key));
+  return param->GetDefaultValue();
+}
+
 void Application::SetDefaultOutputPixelType(std::string const& key, ImagePixelType type)
 {
   auto param = downcast_check<OutputImageParameter>(GetParameterByKey(key));
@@ -1185,6 +1234,18 @@ void Application::SetMaximumParameterFloatValue(std::string const& key, float va
   param->SetMaximumValue(value);
 }
 
+void Application::SetMinimumParameterDoubleValue(std::string const& key, double value)
+{
+  auto param = downcast_check<DoubleParameter>(GetParameterByKey(key));
+  param->SetMinimumValue(value);
+}
+
+void Application::SetMaximumParameterDoubleValue(std::string const& key, double value)
+{
+  auto param = downcast_check<DoubleParameter>(GetParameterByKey(key));
+  param->SetMaximumValue(value);
+}
+
 void Application::SetListViewSingleSelectionMode(std::string const& key, bool status)
 {
   auto param = downcast_check<ListViewParameter>(GetParameterByKey(key));
@@ -1246,6 +1307,11 @@ float Application::GetParameterFloat(std::string const& key) const
   return GetParameterByKey(key)->ToFloat();
 }
 
+double Application::GetParameterDouble(std::string const& key) const
+{
+  return GetParameterByKey(key)->ToDouble();
+}
+
 std::string Application::GetParameterString(std::string const& key) const
 {
   return GetParameterByKey(key)->ToString();
@@ -1395,27 +1461,34 @@ std::vector<std::pair<std::string, std::string>> Application::GetOutputParameter
 {
   std::vector<std::pair<std::string, std::string>> res;
   std::vector<std::string> paramList = GetParametersKeys(true);
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    Parameter*    param = GetParameterByKey(*it);
-    ParameterType type  = GetParameterType(*it);
+    Parameter*    param = GetParameterByKey(key);
+    ParameterType type  = GetParameterType(key);
 
     if (type != ParameterType_Group)
     {
-      if (param->GetRole() == Role_Output && IsParameterEnabled(*it))
+      if (param->GetRole() == Role_Output && IsParameterEnabled(key))
       {
         std::pair<std::string, std::string> keyVal;
-        keyVal.first = (*it);
+        keyVal.first = (key);
         if (type == ParameterType_Float)
         {
           std::ostringstream oss;
           oss << std::setprecision(10);
-          oss << GetParameterFloat(*it);
+          oss << GetParameterFloat(key);
+          keyVal.second = oss.str();
+        }
+        else if (type == ParameterType_Double)
+        {
+          std::ostringstream oss;
+          oss << std::setprecision(19);
+          oss << GetParameterDouble(key);
           keyVal.second = oss.str();
         }
         else
         {
-          keyVal.second = GetParameterAsString(*it);
+          keyVal.second = GetParameterAsString(key);
         }
         res.push_back(keyVal);
       }
@@ -1430,10 +1503,10 @@ bool Application::IsApplicationReady()
   bool ready = true;
 
   std::vector<std::string> paramList = GetParametersKeys(true);
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
     // Check all parameters
-    if (IsParameterMissing(*it))
+    if (IsParameterMissing(key))
     {
       ready = false;
       break;
@@ -1751,9 +1824,8 @@ void Application::PropagateConnectMode(bool isMem)
   m_ExecuteDone                              = false;
   std::vector<std::string>         paramList = GetParametersKeys(true);
   std::unordered_set<Application*> targetApps;
-  for (std::vector<std::string>::const_iterator it = paramList.begin(); it != paramList.end(); ++it)
+  for (auto const & key : paramList)
   {
-    std::string          key      = *it;
     Parameter*           param    = GetParameterByKey(key);
     InputImageParameter* imgParam = dynamic_cast<InputImageParameter*>(param);
 
@@ -1793,5 +1865,22 @@ bool Application::IsExecuteDone()
 {
   return m_ExecuteDone;
 }
+
+bool Application::IsMultiWritingEnabled()
+{
+  return m_MultiWriting;
+}
+
+void Application::EnableInPrivateDo()
+{
+  m_IsInPrivateDo = true;
+
+}
+
+void Application::DisableInPrivateDo()
+{
+  m_IsInPrivateDo = false;
+}
+
 }
 }
